'use strict';

var worker_threads = require('worker_threads');
var pluginCommonjs = require('@rollup/plugin-commonjs');
var pluginJson = require('@rollup/plugin-json');
var pluginReplace = require('@rollup/plugin-replace');
var pluginNodeResolve = require('@rollup/plugin-node-resolve');
var pirates = require('pirates');
var fs = require('fs');
var isModule = require('is-module');
var module$1 = require('module');
var path = require('path');
var sucrase = require('sucrase');
var mime = require('mime');
var rimraf = require('rimraf');
var send = require('send');
var rollup$1 = require('rollup');
var pluginVirtual = require('@rollup/plugin-virtual');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var worker_threads__default = /*#__PURE__*/_interopDefaultLegacy(worker_threads);
var pluginCommonjs__default = /*#__PURE__*/_interopDefaultLegacy(pluginCommonjs);
var pluginJson__default = /*#__PURE__*/_interopDefaultLegacy(pluginJson);
var pluginReplace__default = /*#__PURE__*/_interopDefaultLegacy(pluginReplace);
var pluginNodeResolve__default = /*#__PURE__*/_interopDefaultLegacy(pluginNodeResolve);
var pirates__default = /*#__PURE__*/_interopDefaultLegacy(pirates);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var isModule__default = /*#__PURE__*/_interopDefaultLegacy(isModule);
var module__default = /*#__PURE__*/_interopDefaultLegacy(module$1);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var sucrase__default = /*#__PURE__*/_interopDefaultLegacy(sucrase);
var mime__default = /*#__PURE__*/_interopDefaultLegacy(mime);
var rimraf__default = /*#__PURE__*/_interopDefaultLegacy(rimraf);
var send__default = /*#__PURE__*/_interopDefaultLegacy(send);
var rollup__default = /*#__PURE__*/_interopDefaultLegacy(rollup$1);
var pluginVirtual__default = /*#__PURE__*/_interopDefaultLegacy(pluginVirtual);

let resolvePlugin = pluginNodeResolve__default['default'];

if ('default' in resolvePlugin) {
  // @ts-ignore
  resolvePlugin = resolvePlugin.default;
}

const plugins = [
  // @ts-ignore
  pluginReplace__default['default']({
    'process.env.NODE_ENV': `"${process.env.NODE_ENV}"` || '"development"',
  }),
  // @ts-ignore
  resolvePlugin({
    mainFields: ['module', 'browser', 'main'],
  }),
  // @ts-ignore
  pluginJson__default['default'](),
  // @ts-ignore
  pluginCommonjs__default['default']({
    sourceMap: false,
  }),
];

/**
 * Retrieve the default Rollup-config
 * @returns { import("rollup").RollupOptions }
 */
var getDefaultRollupConfig = function getDefaultRollupConfig() {
  return {
    // Only bundle local package files
    external: (id, parent, isResolved) => {
      // Skip if already handled by plugin
      if (isResolved || (parent && parent.includes('?commonjs-proxy'))) {
        return false;
      }
      return /^[^./\0]/.test(id);
    },
    plugins: plugins.slice(),
    treeshake: false,
    output: {
      format: 'es',
      sourcemap: false,
    },
  };
};

var defaultRollupConfig = {
	getDefaultRollupConfig: getDefaultRollupConfig
};

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function createCommonjsModule(fn, basedir, module) {
	return module = {
	  path: basedir,
	  exports: {},
	  require: function (path, base) {
      return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
    }
	}, fn(module, module.exports), module.exports;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

var brokenNamedExports = {
  react: [
    'Children',
    'Component',
    'Fragment',
    'Profiler',
    'PureComponent',
    'StrictMode',
    'Suspense',
    '__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED',
    'cloneElement',
    'createContext',
    'createElement',
    'createFactory',
    'createRef',
    'forwardRef',
    'isValidElement',
    'lazy',
    'memo',
    'useCallback',
    'useContext',
    'useDebugValue',
    'useEffect',
    'useImperativeHandle',
    'useLayoutEffect',
    'useMemo',
    'useReducer',
    'useRef',
    'useState',
    'version',
  ],
  'react-dom': [
    '__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED',
    'createPortal',
    'findDOMNode',
    'flushSync',
    'hydrate',
    'render',
    'unmountComponentAtNode',
    'unstable_batchedUpdates',
    'unstable_createPortal',
    'unstable_renderSubtreeIntoContainer',
    'version',
  ],
  'react-is': ['isContextConsumer', 'isValidElementType'],
};

const { isMainThread } = worker_threads__default['default'];





const DIR = '.dvlp';
const JS_MIME_TYPES = {
  'application/javascript': ['js', 'jsx', 'ts', 'tsx'],
};
const TESTING =
  process.env.NODE_ENV === 'dvlptest' || process.env.CI != undefined;
// Replaced during build
const VERSION = commonjsGlobal.$VERSION || 'dev';

const bundleDirName = `${path__default['default'].join(DIR, `bundle-${VERSION}`)}`;
const bundleDir = path__default['default'].resolve(bundleDirName);
const port = process.env.PORT ? Number(process.env.PORT) : 8080;

mime__default['default'].define(JS_MIME_TYPES, true);
send__default['default'].mime.define(JS_MIME_TYPES, true);

// Guard against multiple workers performing the same file operations
if (isMainThread) {
  const dir = path__default['default'].resolve(DIR);
  const bundleDirExists = fs__default['default'].existsSync(bundleDir);
  const dirExists = fs__default['default'].existsSync(dir);
  const rm = dirExists && !bundleDirExists;

  if (rm) {
    const contents = fs__default['default'].readdirSync(dir).map((item) => path__default['default'].resolve(dir, item));

    for (const item of contents) {
      // Delete all subdirectories
      if (fs__default['default'].statSync(item).isDirectory()) {
        rimraf__default['default'].sync(item);
      }
    }
  }
  if (!dirExists) {
    fs__default['default'].mkdirSync(dir);
  }
  if (!bundleDirExists) {
    fs__default['default'].mkdirSync(bundleDir);
  }

  if (TESTING) {
    process.on('exit', () => {
      rimraf__default['default'].sync(dir);
    });
  }
}

/**
 * @type { Config }
 */
var config = {
  applicationPort: port,
  brokenNamedExportsPackages: brokenNamedExports,
  bundleDir,
  bundleDirName,
  directories: [],
  // Ordered to trigger transpiling if necessary
  extensionsByType: {
    css: ['.pcss', '.sass', '.scss', '.less', '.styl', '.stylus', '.css'],
    html: [
      '.nunjs',
      '.nunjucks',
      '.hbs',
      '.handlebars',
      '.dust',
      '.html',
      '.htm',
    ],
    js: ['.ts', '.tsx', '.jsx', '.mjs', '.js', '.json'],
  },
  latency: 50,
  maxAge: '10m',
  port,
  reloadEndpoint: '/dvlpreload',
  reloadPort: 0,
  testing: TESTING,
  typesByExtension: {
    '.css': 'css',
    '.pcss': 'css',
    '.sass': 'css',
    '.scss': 'css',
    '.less': 'css',
    '.styl': 'css',
    '.stylus': 'css',
    '.html': 'html',
    '.htm': 'html',
    '.nunjs': 'html',
    '.nunjucks': 'html',
    '.hbs': 'html',
    '.handlebars': 'html',
    '.dust': 'html',
    '.js': 'js',
    '.mjs': 'js',
    '.json': 'js',
    '.jsx': 'js',
    '.ts': 'js',
    '.tsx': 'js',
  },
  version: VERSION,
};

var module_1 = createCommonjsModule(function (module) {

const { addHook } = pirates__default['default'];

// @ts-ignore

// Work around @rollup/plugin-commonjs dynamic require
// @ts-ignore
const loadModule = module__default['default']._load;


const { extensionsByType } = config;

/** @type { () => void } */
let revertHook;

module.exports = {
  isModule,
  importModule,
};

/**
 * Determine if filePath or code is es module
 *
 * @param { string } filePathOrCode
 * @returns { boolean }
 */
function isModule(filePathOrCode) {
  if (
    !filePathOrCode.includes('\n') &&
    extensionsByType.js.includes(path__default['default'].extname(filePathOrCode))
  ) {
    filePathOrCode = fs__default['default'].readFileSync(filePathOrCode, 'utf8');
  }
  return isModule__default['default'](filePathOrCode);
}

/**
 * Import esm/cjs module, transpiling if necessary (via require hook)
 *
 * @param { string } modulePath
 * @param { (filePath: string) => string | undefined } transform
 * @returns { any }
 */
function importModule(modulePath, transform = (filePath) => undefined) {
  if (revertHook !== undefined) {
    revertHook();
  }

  revertHook = addHook(
    (code, filePath) => {
      const transformed = transform(filePath);

      if (transformed !== undefined) {
        code = transformed;
      }

      if (!isModule(code)) {
        return code;
      }

      return sucrase__default['default'].transform(code, {
        transforms: ['imports'],
        filePath,
      }).code;
    },
    {
      exts: extensionsByType.js,
      ignoreNodeModules: false,
    },
  );
  let mod = loadModule(modulePath, module, false);

  // Return default if only exported key
  if ('default' in mod && Object.keys(mod).length === 1) {
    mod = mod.default;
  }

  return mod;
}
});

const {
  isMainThread: isMainThread$1,
  parentPort,
  Worker,
  workerData,
} = worker_threads__default['default'];
const {
  getDefaultRollupConfig: getDefaultRollupConfig$1,
} = defaultRollupConfig;
const { importModule } = module_1;
const { rollup } = rollup__default['default'];
// @ts-ignore


class BundleWorker {
  /**
   * Constructor
   *
   * @param { string } [rollupConfigPath]
   */
  constructor(rollupConfigPath) {
    const defaultConfig = getDefaultRollupConfig$1();

    this.rollupConfig = rollupConfigPath
      ? mergeRollupConfig(defaultConfig, importModule(rollupConfigPath))
      : defaultConfig;
    this.worker;

    if (isMainThread$1) {
      // Load this file in Worker thread, passing `rollupConfigPath` as data
      this.worker = new Worker(__filename, { workerData: rollupConfigPath });
    }
  }

  /**
   * Bundle file at "inputPath"
   *
   * @param { string } inputPath
   * @param { string } outputPath
   * @param { string } sourcePrefix
   * @param { Array<string> } [namedExports]
   */
  async bundle(inputPath, outputPath, sourcePrefix, namedExports) {
    if (this.worker) {
      return new Promise((resolve, reject) => {
        if (this.worker) {
          this.worker.postMessage({
            inputPath,
            outputPath,
            sourcePrefix,
            namedExports,
          });
          this.worker.once('message', (err) => {
            err ? reject(err) : resolve();
          });
        } else {
          reject(Error('No bundle worker instance found'));
        }
      });
    }

    const banner = sourcePrefix + inputPath;
    const parsedOptions = parseRollupOptions(banner, this.rollupConfig);
    const inputOptions = {
      ...parsedOptions.input,
      input: inputPath,
    };
    const outputOptions = {
      ...parsedOptions.output,
      file: outputPath,
    };

    if (namedExports && inputOptions.plugins) {
      inputOptions.plugins.unshift(
        pluginVirtual__default['default']({
          entry: `import entry from '${inputPath}';
        export default entry;
        export {${namedExports.join(', ')}} from '${inputPath}';`,
        }),
      );
      inputOptions.input = 'entry';
    }

    const bundled = await rollup(inputOptions);
    await bundled.write(outputOptions);
  }

  /**
   * Destroy instance
   *
   * @returns { Promise<void> }
   */
  destroy() {
    if (this.worker) {
      // @ts-ignore
      return this.worker.terminate();
    }
    return Promise.resolve();
  }
}

var bundleWorker = BundleWorker;

// Create instance if run in Worker thread
if (!isMainThread$1 && parentPort) {
  /** @type { string | undefined } */
  const rollupConfigPath = workerData;
  const bundleWorker = new BundleWorker(rollupConfigPath);

  parentPort.on('message', async (
    /** @type { BundleWorkerMessage } */ message,
  ) => {
    const { inputPath, outputPath, sourcePrefix, namedExports } = message;

    try {
      await bundleWorker.bundle(
        inputPath,
        outputPath,
        sourcePrefix,
        namedExports,
      );
      parentPort.postMessage(false);
    } catch (err) {
      parentPort.postMessage(err);
    }
  });
}

/**
 * Merge user rollup.config with default
 *
 * @param { RollupOptions } defaultConfig
 * @param { RollupOptions } newConfig
 * @returns { RollupOptions }
 */
function mergeRollupConfig(defaultConfig, newConfig) {
  const {
    output: requiredOutput,
    plugins: defaultPlugins = [],
    ...defaultOptions
  } = defaultConfig;
  const { output, plugins = [], ...options } = newConfig;
  /** @type { { [name: string]: import('rollup').Plugin }}  */
  const newPluginsByName = plugins.reduce((newPluginsByName, plugin) => {
    // @ts-ignore
    newPluginsByName[plugin.name] = plugin;
    return newPluginsByName;
  }, {});
  let mergedPlugins = [];

  // Replace default plugin with new if it has the same name
  for (const plugin of defaultPlugins) {
    const { name } = plugin;
    mergedPlugins.push(
      name in newPluginsByName ? newPluginsByName[name] : plugin,
    );
  }

  return {
    ...defaultOptions,
    ...options,
    plugins: mergedPlugins,
    output: { ...output, ...requiredOutput },
  };
}

/**
 * Parse Rollup options
 *
 * @param { string } banner
 * @param { RollupOptions } options
 * @returns { { input: RollupInputOptions, output: RollupOutputOptions } }
 */
function parseRollupOptions(banner, options) {
  if (!options) {
    return { input: {}, output: { banner } };
  }

  let { input, treeshake, output = {}, watch, ...inputOverride } = options;

  if (Array.isArray(output)) {
    output = output[0];
  }

  const { file, format, sourcemap, ...outputOverride } = output;

  if ('banner' in outputOverride) {
    outputOverride.banner = banner + outputOverride.banner;
  } else {
    outputOverride.banner = banner;
  }

  // Shallow copy plugins
  if (inputOverride.plugins) {
    inputOverride.plugins = [...inputOverride.plugins];
  }

  return { input: inputOverride, output: outputOverride };
}

module.exports = bundleWorker;
